﻿const fs = require('fs');
const path = require('path');
const sizeOf = require('image-size');
const tmpl_less = `@%NAME%-width:%WIDTH%px;
@%NAME%-height:%HEIGHT%px;
@%NAME%-image:url("%PATH%");
.%NAME%(){
	background-image:url("%PATH%");
	background-size:%WIDTH%px %HEIGHT%px;
}
.%NAME%--block(){
	background-image:url("%PATH%");
	background-size:%WIDTH%px %HEIGHT%px;
	width:%WIDTH%px;
	height:%HEIGHT%px;
}
`;
const tmpl_sass = `$%NAME%-width:%WIDTH%px;
$%NAME%-height:%HEIGHT%px;
$%NAME%-image:url("%PATH%");
@mixin %NAME%(){
	background-image:url("%PATH%");
	background-size:%WIDTH%px %HEIGHT%px;
}
@mixin %NAME%--block(){
	background-image:url("%PATH%");
	background-size:%WIDTH%px %HEIGHT%px;
	width:%WIDTH%px;
	height:%HEIGHT%px;
}
`;
let walk = function(dir, done) {
	let results = [];
	fs.readdir(dir, function(err, list) {
		if (err) return done(err);
		let pending = list.length;
		if (!pending) return done(null, results);
		list.forEach(function(file) {
			file = path.resolve(dir, file);
			fs.stat(file, function(err, stat) {
				if (stat && stat.isDirectory()) {
					walk(file, function(err, res) {
						results = results.concat(res);
						if (!--pending) done(null, results);
					});
				} else {
					results.push(file);
					if (!--pending) done(null, results);
				}
			});
		});
	});
};
let generate = function(option){
	let isRetina = false;
	let pathVar;
	let outFile;
	let type;
	if(typeof option === 'object'){
		if(option.isRetina){
			isRetina = true;
		}
		if(option.pathVar){
			pathVar = option.pathVar;
		}
		if(option.outFile){
			outFile = option.outFile;
		}
		if(option.type){
			type = option.type;
		}
	}
	let filename;
	let tmpl;
	let interpolation;
	if(!type || type==='less'){
		filename = outFile?outFile:'images.less';
		tmpl = tmpl_less;
		interpolation = pathVar?'@{'+ pathVar +'}':'';
	} else if(type==='sass'){
		filename = outFile?outFile:'images.sass';
		tmpl = tmpl_sass;
		interpolation = pathVar?'#{$'+ pathVar +'}':'';
	}
	let resText = '';
	let images = [];
	walk(process.cwd(), function(err, results) {
		if (err) throw err;
		results.filter(function(file){
			return fs.statSync(file).isFile() && /.*\.(png|jpg|jpeg|gif)$/.test(file); // 絞り込み
		}).sort().forEach(function (file) {
			let imgData = sizeOf(file);
			let w = imgData.width;
			let h = imgData.height;
			if(isRetina){
				w = Math.ceil(w/2);
				h = Math.ceil(h/2);
			}
			let basename = path.relative(process.cwd(), file);
			let matches = basename.match(/(.+)\.(png|jpg|jpeg|gif)$/i);
			let name = matches[1].split(path.sep).join('__');
			let res = tmpl.replace(/%NAME%/g,name)
			.replace(/%WIDTH%/g,w)
			.replace(/%HEIGHT%/g,h);
			resText += res.replace(/%PATH%/g,interpolation+basename.split(path.sep).join('/'));
		});
		fs.writeFileSync(filename,resText);
	});
}
module.exports = generate;